@@ -18,6 +18,8 @@
  * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "sj-metadata"
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
@@ -35,6 +37,16 @@
 #include "sj-structures.h"
 #include "sj-error.h"
 
+typedef char Mbid[40];
+
+typedef struct {
+  gint count;
+  gchar *id;
+  gchar *mcn;
+  gchar *url;
+  Mbid  *release_ids;
+} DiscDetails;
+
 static char language[3];
 
 #define GET(field, function, obj) {						\
@@ -107,6 +119,15 @@ sj_mb5_album_details_dump (AlbumDetails *details)
 #define sj_mb5_album_details_dump(...)
 #endif
 
+static void
+disc_details_free (DiscDetails *disc)
+{
+  g_free (disc->id);
+  g_free (disc->mcn);
+  g_free (disc->url);
+  g_free (disc->release_ids);
+  g_free (disc);
+}
 
 static void
 parse_artist_aliases (Mb5Artist   artist,
@@ -181,11 +202,12 @@ print_musicbrainz_query_error (SjMetadataMusicbrainz5 *self,
   len =  mb5_query_get_lasterrormessage (priv->mb, NULL, 0) + 1;
   message = g_malloc (len);
   mb5_query_get_lasterrormessage (priv->mb, message, len);
-  g_warning ("No Musicbrainz metadata for %s %s, http code %d, %s",
+  g_info ("No Musicbrainz metadata for %s %s, http code %d, %s",
              entity, id, code, message);
   g_free (message);
 }
 
+static gint64 last_query_time;
 
 static Mb5Metadata
 query_musicbrainz (SjMetadataMusicbrainz5  *self,
@@ -197,23 +219,172 @@ query_musicbrainz (SjMetadataMusicbrainz5  *self,
 {
   Mb5Metadata metadata;
   char *inc[] = { "inc" };
+  gint64 delay = 4 * G_USEC_PER_SEC;
+  int count = 0;
   SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+  gint64 t;
 
-  if (g_cancellable_set_error_if_cancelled (cancellable, error))
-    return NULL;
+  g_info ("Querying %s %s", entity, id);
+
+  while (TRUE) {
+    if (g_cancellable_set_error_if_cancelled (cancellable, error))
+      return NULL;
+
+    t = g_get_monotonic_time ();
+    while (t - last_query_time < G_USEC_PER_SEC) {
+      g_usleep (t - last_query_time + G_USEC_PER_SEC);
+      t = g_get_monotonic_time ();
+    }
+
+    if (includes == NULL)
+      metadata = mb5_query_query (priv->mb, entity, id, "",
+                                  0, NULL, NULL);
+    else
+      metadata = mb5_query_query (priv->mb, entity, id, "",
+                                  1, inc, &includes);
+
+    if (metadata != NULL || ++count == 5 ||
+        mb5_query_get_lasthttpcode (priv->mb) != 503)
+      break;
+    g_info ("Retrying %d sleeping for %.2g",
+            count,
+            (double)(delay)/(double)(G_USEC_PER_SEC));
+
+    g_usleep (delay);
+    if (delay < 30 * G_USEC_PER_SEC) {
+      delay *= 45; /* Increase delay by ~2√2 */
+      delay /= 16; /* Gives 4, 11, 30 */
+    }
+  }
 
-  if (includes == NULL)
-    metadata = mb5_query_query (priv->mb, entity, id, "",
-                                0, NULL, NULL);
-  else
-    metadata = mb5_query_query (priv->mb, entity, id, "",
-                                1, inc, &includes);
   if (metadata == NULL)
     print_musicbrainz_query_error (self, entity, id);
 
+  last_query_time = g_get_monotonic_time ();
   return metadata;
 }
 
+static Mb5Metadata
+get_disc_md (SjMetadataMusicbrainz5  *self,
+             DiscDetails             *disc_data,
+             GCancellable            *cancellable,
+             GError                 **error)
+{
+  const char *discid;
+  DiscId disc;
+  Mb5Metadata disc_md = NULL;
+  SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+
+  if (sj_metadata_helper_check_media (priv->cdrom, error) == FALSE) {
+    return NULL;
+  }
+
+  disc = discid_new ();
+  if (disc == NULL)
+    return NULL;
+  if (discid_read_sparse (disc, priv->cdrom, DISCID_FEATURE_MCN) == 0)
+    goto out;
+
+  if (g_getenv("MUSICBRAINZ_FORCE_DISC_ID")) {
+    discid = g_getenv("MUSICBRAINZ_FORCE_DISC_ID");
+  } else {
+    discid = discid_get_id (disc);
+  }
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    goto out;
+
+  disc_md = query_musicbrainz (self, "discid", discid, NULL, cancellable, error);
+  disc_data->url = g_strdup (discid_get_submission_url (disc));
+  disc_data->mcn = g_strdup (discid_get_mcn (disc));
+  disc_data->id = g_strdup (discid);
+  g_info ("Disc id %s\nSubmission URL %s\nDisc MCN %s",
+          disc_data->id,
+          disc_data->url,
+          disc_data->mcn);
+ out:
+  discid_free (disc);
+  return disc_md;
+}
+
+static gboolean
+mcn_matches_barcode (const char *mcn,
+                     const char *barcode)
+{
+  if (mcn == NULL || barcode == NULL)
+    return FALSE;
+
+  /* The MCN should match an EAN barcode (13 digits)
+   * or an UPC barcode (12 digits) with a leading '0'.
+   */
+  gsize len = strlen (barcode);
+  if (len == 12) /* UPC barcode - skip leading '0' */
+    return *mcn == '0' && strcmp (mcn + 1, barcode) == 0;
+  else if (len == 13) /* EAN barcode */
+    return strcmp (mcn, barcode) == 0;
+  else /* Unknown/invalid barcode */
+    return FALSE;
+}
+
+static void
+filter_releases (Mb5Metadata  disc_md,
+                 DiscDetails *disc)
+{
+  int i, j, count;
+  Mb5ReleaseList releases;
+
+  releases = mb5_disc_get_releaselist (mb5_metadata_get_disc (disc_md));
+  count = disc->count = mb5_release_list_size (releases);
+  if (disc->count == 0)
+    return;
+
+  disc->release_ids = g_new0 (Mbid, disc->count);
+  for (i = 0, j = 0; i < count; i++) {
+    Mb5Release release;
+    gchar barcode[16];
+
+    release = mb5_release_list_item (releases, i);
+    if (release == NULL) {
+      disc->count--;
+      continue;
+    }
+
+    mb5_release_get_id (release, disc->release_ids[j], sizeof(Mbid));
+    mb5_release_get_barcode (release, barcode, sizeof(barcode));
+    if (mcn_matches_barcode (disc->mcn, barcode)) {
+      disc->count = 1;
+      strcpy (disc->release_ids[0], disc->release_ids[j]);
+      break;
+    }
+    j++;
+  }
+}
+
+static DiscDetails*
+get_disc_details (SjMetadataMusicbrainz5  *self,
+                  GCancellable            *cancellable,
+                  GError                 **error)
+{
+  Mb5Metadata disc_md;
+  DiscDetails *disc;
+
+  disc = g_new0 (DiscDetails, 1);
+  disc_md = get_disc_md (self, disc, cancellable, error);
+
+  if (*error != NULL) {
+    disc_details_free (disc);
+    return NULL;
+  }
+
+  if (disc_md == NULL)
+    return disc;
+
+  filter_releases (disc_md, disc);
+
+  mb5_metadata_delete (disc_md);
+
+  return disc;
+}
+
 static ArtistDetails*
 query_artist (SjMetadataMusicbrainz5  *self,
               const gchar             *id,
@@ -266,7 +437,7 @@ get_artist_list (SjMetadataMusicbrainz5 *self,
     if (artist != NULL) {
       artist_credit->details = make_artist_details (self, artist);
     } else {
-      g_warning ("no Mb5Artist associated with Mb5NameCredit, falling back to Mb5NameCredit::name");
+      g_info ("no Mb5Artist associated with Mb5NameCredit, falling back to Mb5NameCredit::name");
       artist_credit->details = g_new0 (ArtistDetails, 1);
       GET (artist_credit->details->name, mb5_namecredit_get_name, name_credit);
     }
@@ -295,7 +466,7 @@ get_artist_info (GList *artists, char **name, char **sortname, char **id)
   }
 
   if (artist_count != 1) {
-      g_warning ("multiple artists");
+      g_info ("multiple artists");
       if (sortname != NULL)
         *sortname = NULL;
       if (id != NULL)
@@ -473,51 +644,11 @@ build_composer_text (GSList  *composers,
 {
   gsize i = 0;
   gsize j = 0;
-  gsize index = 0; /* Which format string to use */
+  guint flags = 0; /* Which components do we have? */
   GSList *sort = NULL;
   gchar  *comp = NULL;
   gchar  *arr  = NULL;
   gchar  *arr_v[4];
-  gchar  *formats[16] = {
-    NULL,
-    NULL,
-    /* Tranlators: This string is used to build the composer tag when
-       a track has a mixture of arrangers, orchestrators and
-       transcribers but no composers */
-    N_("arr. %s"),
-    /* Tranlators: This string is used to build the composer tag when a
-       track has composers and arrangers, or a composer and a mixture
-       of arrangers, orchestrators and transcribers */
-    N_("%s arr. %s"),
-    /* Tranlators: This string is used to build the composer tag
-       when a track has orchestrators but no composer */
-    N_("orch. %s"),
-    /* Tranlators: This string is used to build the composer tag
-       when a track has composers and orchestrators */
-    N_("%s orch. %s"),
-    /* arrangers and orchestrators but no composers */
-    "arr. %s",
-    /* composers, arrangers and orchestrators */
-    "%s arr. %s",
-    /* Tranlators: This string is used to build the composer tag
-       when a track has a transcribers but no composer */
-    N_("trans. %s"),
-    /* Tranlators: This string is used to build the composer tag
-       when a track has composers and transcribers */
-    N_("%s trans. %s"),
-    /* arrangers and transcribers but no composers */
-    "arr. %s",
-    /* composers, arrangers and transcribers */
-    "%s arr. %s",
-    /* orchestrators and transcribers but no composer */
-    "arr. %s",
-    /* composers, orchestrators and transcribers */
-    "%s arr. %s",
-    /* arrangers, orchestrators and transcribers but no composers */
-    "arr. %s",
-    /* composers, arrangers, orchestrators and transcribers */
-    "%s arr. %s"
-  };
 
   enum {
     HAVE_COMPOSERS     = 1 << 0,
@@ -532,22 +663,22 @@ build_composer_text (GSList  *composers,
   *name = NULL;
 
   if (composers != NULL) {
-    index |= HAVE_COMPOSERS;
+    flags |= HAVE_COMPOSERS;
     comp = concatenate_composers (composers);
     sort = composers;
   }
   if (arrangers != NULL) {
-    index |= HAVE_ARRANGERS;
+    flags |= HAVE_ARRANGERS;
     arr_v[i++] = concatenate_composers (arrangers);
     sort = arrangers;
   }
   if (orchestrators != NULL) {
-    index |= HAVE_ORCHESTRATORS;
+    flags |= HAVE_ORCHESTRATORS;
     arr_v[i++] = concatenate_composers (orchestrators);
     sort = orchestrators;
   }
   if (transcribers != NULL) {
-    index |= HAVE_TRANSCRIBERS;
+    flags |= HAVE_TRANSCRIBERS;
     arr_v[i++] = concatenate_composers (transcribers);
     sort = transcribers;
   }
@@ -556,25 +687,43 @@ build_composer_text (GSList  *composers,
   if (i > 0)
     arr = g_strjoinv (", ", arr_v);
 
-  if (index & HAVE_COMPOSERS) {
-    if (index & ~HAVE_COMPOSERS) {
-      *name = g_strdup_printf (gettext (formats[index]), comp, arr);
+  if (flags & HAVE_COMPOSERS) {
+    if (flags & HAVE_ARRANGERS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has composers and arrangers, or a composer and a
+         mixture of arrangers, orchestrators and transcribers */
+      *name = g_strdup_printf (_("%s arr. %s"), comp, arr);
+    } else if (flags & HAVE_ORCHESTRATORS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has composers and orchestrators */
+      *name = g_strdup_printf (_("%s orch. %s"), comp, arr);
+    } else if (flags & HAVE_TRANSCRIBERS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has composers and transcribers */
+      *name = g_strdup_printf (_("%s trans. %s"), comp, arr);
     } else {
+      /* Only composers */
       *name = g_strdup (comp);
     }
   } else {
-    if (index & ~HAVE_COMPOSERS) {
-      *name = g_strdup_printf (gettext (formats[index]), arr);
+    if (flags & HAVE_ARRANGERS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has a mixture of arrangers, orchestrators and
+         transcribers but no composers */
+      *name = g_strdup_printf (_("arr. %s"), arr);
+    } else if (flags & HAVE_ORCHESTRATORS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has orchestrators but no composer */
+      *name = g_strdup_printf (_("orch. %s"), arr);
+    } else if (flags & HAVE_TRANSCRIBERS) {
+      /* Translators: This string is used to build the composer tag
+         when a track has a transcribers but no composer */
+      *name = g_strdup_printf (_("trans. %s"), arr);
     }
   }
 
-  switch (index) {
-  case HAVE_COMPOSERS:
-  case HAVE_ARRANGERS:
-  case HAVE_ORCHESTRATORS:
-  case HAVE_TRANSCRIBERS:
+  if (flags)
     *sortname = get_sortname (sort);
-  }
 
   while (j < i)
     g_free (arr_v[j++]);
@@ -718,6 +867,7 @@ fill_tracks_from_medium (SjMetadataMusicbrainz5  *self,
   Mb5TrackList track_list;
   GList *tracks;
   unsigned int i;
+  gboolean skip_data_tracks;
   char buffer[512]; /* for the GET() macro */
 
   track_list = mb5_medium_get_tracklist (medium);
@@ -727,31 +877,55 @@ fill_tracks_from_medium (SjMetadataMusicbrainz5  *self,
   album->number = mb5_track_list_size (track_list);
 
   tracks = NULL;
+  skip_data_tracks = TRUE;
 
   for (i = 0; i < mb5_track_list_size (track_list); i++) {
     Mb5Track mbt;
     Mb5ArtistCredit credit;
     Mb5Recording recording;
     TrackDetails *track;
+    gchar *title;
+    gint track_offset;
 
     mbt = mb5_track_list_item (track_list, i);
     if (!mbt)
       continue;
 
-    track = g_new0 (TrackDetails, 1);
+    recording = mb5_track_get_recording (mbt);
+    /*
+     * title points to a string on the stack so set it to NULL to
+     * prevent the GET macro from trying to free it
+     */
+    title = NULL;
+    GET (title, mb5_track_get_title, mbt);
+    if (title == NULL && recording != NULL)
+      GET (title, mb5_recording_get_title, recording);
 
+    /*
+     * Skip data tracks at the start of the disc
+     * https://musicbrainz.org/doc/Style/Unknown_and_untitled/Special_purpose_track_title
+     */
+    if (skip_data_tracks) {
+      if (g_strcmp0 (title, "[data track]") == 0) {
+        continue;
+      } else {
+        skip_data_tracks = FALSE;
+        track_offset = mb5_track_get_position (mbt) - 1;
+      }
+    }
+
+    track = g_new0 (TrackDetails, 1);
+    track->title = g_strdup (title);
     track->album = album;
 
-    track->number = mb5_track_get_position (mbt);
+    track->number = mb5_track_get_position (mbt) - track_offset;
 
     /* Prefer metadata from Mb5Track over metadata from Mb5Recording
      * https://bugzilla.gnome.org/show_bug.cgi?id=690903#c8
      */
     track->duration = mb5_track_get_length (mbt) / 1000;
-    GET (track->title, mb5_track_get_title, mbt);
     credit = mb5_track_get_artistcredit (mbt);
 
-    recording = mb5_track_get_recording (mbt);
     if (recording != NULL) {
       GET (track->track_id, mb5_recording_get_id, recording);
       fill_recording_relations (self, recording, track, cancellable, error);
@@ -762,8 +936,6 @@ fill_tracks_from_medium (SjMetadataMusicbrainz5  *self,
 
       if (track->duration == 0)
         track->duration = mb5_recording_get_length (recording) / 1000;
-      if (track->title == NULL)
-        GET (track->title, mb5_recording_get_title, recording);
       if (credit == NULL)
           credit = mb5_recording_get_artistcredit (recording);
     }
@@ -917,30 +1089,6 @@ make_album_from_release (SjMetadataMusicbrainz5  *self,
   return album;
 }
 
-static gboolean
-album_barcode_matches_discid_mcn (const char *barcode,
-                                  DiscId     *disc)
-{
-  const char *mcn;
-
-  if (barcode == NULL || !discid_has_feature(DISCID_FEATURE_MCN))
-    return FALSE;
-
-  mcn = discid_get_mcn (disc);
-  if (mcn == NULL)
-    return FALSE;
-  /* The MCN should match an EAN barcode (13 digits)
-   * or an UPC barcode (12 digits) with a leading '0'.
-   */
-  gsize len = strlen (barcode);
-  if (len == 12) /* UPC barcode - skip leading '0' */
-    return *mcn == '0' && strcmp (mcn + 1, barcode) == 0;
-  else if (len == 13) /* EAN barcode */
-    return strcmp (mcn, barcode) == 0;
-  else /* Unknown/invalid barcode */
-    return FALSE;
-}
-
 /*
  * Virtual methods
  */
@@ -951,149 +1099,100 @@ mb5_list_albums (SjMetadata    *metadata,
                  GError       **error)
 {
   SjMetadataMusicbrainz5 *self;
-  SjMetadataMusicbrainz5Private *priv;
   GList *albums = NULL;
-  Mb5ReleaseList releases;
-  Mb5Release release;
-  const char *discid = NULL;
-  char *tmp_url = NULL;
+  DiscDetails *disc;
   char buffer[1024];
   int i;
-  DiscId disc = NULL;
-  gboolean mcn_matches = FALSE;
 
   g_return_val_if_fail (SJ_IS_METADATA_MUSICBRAINZ5 (metadata), NULL);
 
   self = SJ_METADATA_MUSICBRAINZ5 (metadata);
-  priv = GET_PRIVATE (SJ_METADATA_MUSICBRAINZ5 (self));
 
-  if (sj_metadata_helper_check_media (priv->cdrom, error) == FALSE) {
-    return NULL;
-  }
-
-  disc = discid_new ();
+  disc = get_disc_details (self, cancellable, error);
+  /* if *error != NULL then disc == NULL */
   if (disc == NULL)
     return NULL;
-  if (discid_read_sparse (disc, priv->cdrom, DISCID_FEATURE_MCN) == 0)
-    goto free_discid;
 
-  if (url != NULL)
-    tmp_url = g_strdup (discid_get_submission_url (disc));
-
-  if (g_getenv("MUSICBRAINZ_FORCE_DISC_ID")) {
-    discid = g_getenv("MUSICBRAINZ_FORCE_DISC_ID");
-  } else {
-    discid = discid_get_id (disc);
-  }
-  if (g_cancellable_set_error_if_cancelled (cancellable, error))
-    goto free_url;
-
-  releases = mb5_query_lookup_discid(priv->mb, discid);
-
-  if (releases == NULL) {
-    print_musicbrainz_query_error (self, "discid", discid);
-    if (url != NULL)
-      *url = tmp_url;
-    goto free_discid;
-  }
-
-  if (mb5_release_list_size (releases) == 0)
-    return NULL;
-
-  for (i = 0; i < mb5_release_list_size (releases) && !mcn_matches; i++) {
+  for (i = 0; i < disc->count; i++) {
     AlbumDetails *album;
-
-    release = mb5_release_list_item (releases, i);
-    if (release) {
-      char *releaseid = NULL;
-      Mb5Release full_release = NULL;
-      Mb5Metadata release_md = NULL;
-      char *includes = "aliases artists artist-credits labels recordings \
+    Mb5Release full_release = NULL;
+    Mb5Metadata release_md = NULL;
+    char *includes = "aliases artists artist-credits labels recordings \
 release-groups url-rels discids recording-level-rels work-level-rels work-rels \
 artist-rels";
 
-      releaseid = NULL;
-      GET(releaseid, mb5_release_get_id, release);
-      /* Inorder to get metadata for artist aliases & work / composer
-       * relationships we need to perform a custom query
-       */
-      release_md = query_musicbrainz (self, "release", releaseid, includes, cancellable, error);
-      g_free (releaseid);
-      if (*error != NULL)
-        goto free_releases;
+    /*
+     * In order to get metadata for artist aliases & work / composer
+     * relationships we need to perform a custom query
+     */
+    release_md = query_musicbrainz (self, "release", disc->release_ids[i], includes, cancellable, error);
+    if (*error != NULL)
+      goto free_releases;
 
-      if (release_md && mb5_metadata_get_release (release_md))
-        full_release = mb5_metadata_get_release (release_md);
-      if (full_release) {
-        Mb5MediumList media;
-        Mb5Metadata metadata = NULL;
-        Mb5ReleaseGroup group;
-        unsigned int j;
+    if (release_md && mb5_metadata_get_release (release_md))
+      full_release = mb5_metadata_get_release (release_md);
+    if (full_release) {
+      Mb5MediumList media;
+      Mb5Metadata group_md = NULL;
+      Mb5ReleaseGroup group;
+      int j;
 
-        group = mb5_release_get_releasegroup (full_release);
-        if (group) {
-          /* The release-group information we can extract from the
-           * lookup_release query doesn't have the url relations for the
-           * release-group, so run a separate query to get these urls
-           */
-          char *releasegroupid = NULL;
-          char *includes = "artists url-rels";
+      group = mb5_release_get_releasegroup (full_release);
+      if (group) {
+        /*
+         * The release-group information we can extract from the
+         * lookup_release query doesn't have the url relations for the
+         * release-group, so run a separate query to get these urls
+         */
+        char *releasegroupid = NULL;
+        char *includes = "artists url-rels";
 
-          GET (releasegroupid, mb5_releasegroup_get_id, group);
-          metadata = query_musicbrainz (self, "release-group", releasegroupid, includes, cancellable, error);
-          g_free (releasegroupid);
-          if (*error != NULL) {
-            mb5_metadata_delete (release_md);
-            goto free_releases;
-          }
+        GET (releasegroupid, mb5_releasegroup_get_id, group);
+        group_md = query_musicbrainz (self, "release-group", releasegroupid, includes, cancellable, error);
+        g_free (releasegroupid);
+        if (*error != NULL) {
+          mb5_metadata_delete (release_md);
+          goto free_releases;
         }
 
-        if (metadata && mb5_metadata_get_releasegroup (metadata))
-          group = mb5_metadata_get_releasegroup (metadata);
+        if (group_md && mb5_metadata_get_releasegroup (group_md))
+          group = mb5_metadata_get_releasegroup (group_md);
 
-        media = mb5_release_media_matching_discid (full_release, discid);
+        media = mb5_release_media_matching_discid (full_release, disc->id);
         for (j = 0; j < mb5_medium_list_size (media); j++) {
           Mb5Medium medium;
+
           medium = mb5_medium_list_item (media, j);
           if (medium) {
             album = make_album_from_release (self, group, full_release, medium, cancellable, error);
             if (*error != NULL) {
-              mb5_metadata_delete (metadata);
+              mb5_metadata_delete (group_md);
               mb5_medium_list_delete (media);
               mb5_metadata_delete (release_md);
               goto free_releases;
             }
 
             album->metadata_source = SOURCE_MUSICBRAINZ;
-            mcn_matches = album_barcode_matches_discid_mcn (album->barcode, disc);
-            if (mcn_matches) {
-              g_list_free_full (albums, (GDestroyNotify) album_details_free);
-              albums = g_list_append (NULL, album);
-              break;
-            }
             albums = g_list_append (albums, album);
           }
         }
-        mb5_metadata_delete (metadata);
+        mb5_metadata_delete (group_md);
         mb5_medium_list_delete (media);
       }
       mb5_metadata_delete (release_md);
     }
   }
+
   if (url != NULL)
-    *url = tmp_url;
-  mb5_release_list_delete (releases);
-  discid_free (disc);
+    *url = g_strdup (disc->url);
+
+  disc_details_free (disc);
   return albums;
 
 
  free_releases:
   g_list_free_full (albums, (GDestroyNotify) album_details_free);
-  mb5_release_list_delete (releases);
- free_url:
-  g_free (tmp_url);
- free_discid:
-  discid_free (disc);
+  disc_details_free (disc);
   return NULL;
 }
 
